/*--------------------------------------------------------------------------*/
/*  Copyright (C) Herbert Oppmann 2003. All Rights Reserved.                */
/*--------------------------------------------------------------------------*/
/*  Project:     MyBCP                                                      */
/*  Filename:    LZHapi.h                                                   */
/*                                                                          */
/*  Description:                                                            */
/*                                                                          */
/*  LZH (de)compression interface                                           */
/*--------------------------------------------------------------------------*/

/* Compression algorithm: known as "-lh5-", */
/* Lempel-Ziv 77 sliding window with static Huffman encoding. */
/* Source taken from Haruhiko Okumura's AR002. */

/*--< typedef >-------------------------------------------------------------*/

typedef unsigned char  uchar;	/*  8 bits or more */
typedef unsigned int   uint;	/* 16 - 32 bits or more */
typedef unsigned short ushort;	/* 16 bits or more */
typedef unsigned long  ulong;	/* 32 bits or more */

/*--< function declarations >-----------------------------------------------*/

/* compression implements */
void encode(void);
/* compression expects */
extern ulong read_uncompressed(uchar *buf, ulong len);
extern void write_compressed(uchar c);

/* decompression implements */
extern void decode(ulong origsize);
/* decompression expects */
extern void write_uncompressed(uchar *p, int n);
extern uchar read_compressed(void);
/* NOTE: read_compressed must return 0 for reads beyond end of stream */
/* without complaining for at least 2 bytes to flush internal bit buffer! */

/* both compression and decompression expect */
void error(char *fmt, ...);

/*--------------------------------------------------------------------------*/
/*  Copyright (C) Herbert Oppmann 2003. All Rights Reserved.                */
/*--------------------------------------------------------------------------*/

