/*--------------------------------------------------------------------------*/
/*  Copyright (C) Herbert Oppmann 2003. All Rights Reserved.                */
/*--------------------------------------------------------------------------*/
/*  Project:     MyBCP                                                      */
/*  Filename:    mybcp.c                                                    */
/*                                                                          */
/*  Description:                                                            */
/*                                                                          */
/*  My 10/10/94 AMIBIOS configuration utility                               */
/*--------------------------------------------------------------------------*/

/*--< define, include >-----------------------------------------------------*/

#define VERSION	"001"
#define DATE	"25 July 2003"

#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
// isdigit macro is somehow broken on Borland C++ 4.51 ?!?
#undef isdigit

#include "LZHapi.h"

#define K 1024

/*--< typedef >-------------------------------------------------------------*/

typedef struct bios_head {
	char	id[8];		// AMIBIOSC
	char	date[8];	// xx/xx/94
	ushort	nMod;
	struct {
		ushort to;
		ushort from;
	} m[1];
} bios_head;

typedef struct module_head {
	ulong compsize;
	ulong origsize;
} module_head;

/*--< variables >-----------------------------------------------------------*/

static uchar bimage[128*K];
static uchar	*p_compressed;
static uchar	*p_uncompressed;
static ulong	nOrigSize, nCompSize;

// TBD: auch mit nur 64KB groen BIOSen zurechtkommen

/*--< function declarations >-----------------------------------------------*/

/*--< functions >-----------------------------------------------------------*/

void error(char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	putc('\n', stderr);
	vfprintf(stderr, fmt, args);
	putc('\n', stderr);
	va_end(args);
	exit(EXIT_FAILURE);
}


uchar read_compressed(void)
{
	if (nCompSize) {
    	nCompSize--;
		return *(p_compressed++);
    } else {
		printf("\n[X]\n");
    	return 0;
    }
}


void write_uncompressed(uchar *p, int n)
{
	memcpy(p_uncompressed, p, n);
	p_uncompressed += n;
}


ulong read_uncompressed(uchar *buf, ulong len)
{
	if (len > nOrigSize) {
    	len = nOrigSize;
    }
    memcpy(buf, p_uncompressed, len);
    p_uncompressed += len;
    nOrigSize -= len;
	return len;
}


void write_compressed(uchar c)
{
	nCompSize++;
	*(p_compressed++) = c;
}

/*
                      AMIBIOS Module Manipulation Utility
 Copyright (C)1998 American Megatrends Inc. All Rights Reserved.  Ver.2.00 Beta

Usage: AMIMM FILE_NAME /<OPTION1> /<OPTION2> .....
Sample: A. Insert EPA logo module into BIOS ROM file.
           -=> AMIMM AMIBIOS.ROM /I LOGO.BIN 31
        B. Use script file to build BIOS ROM file.
           -=> AMIMM AMIBIOS.MM /B

<Options>
         /?[+]
            Help. [+] -> More information.
         /B[Boot_Block_Size]
            Use script file to build BIOS ROM file.
            [Boot_Block_Size] -> 0 - 7 = 8 - 64 Kb, Default is 8 Kb.
         /D <Type#> [<Instance#>]
            Delete module.
         /E <Module Filename> <Type#> [<Instance#>]
            Extract module.
         /I <Module Filename> <Type#> [<Position#>] [/U] [/M <M#1> <M#2>]
            Insert module.
         /L [/P]
            Display BIOS modules information. [/P] -> Pause if module too many.
         /R <Module Filename> <Type#> [<Instance#>] [/U] [/M <M#1> <M#2>]
            Replace module.

Type# is Module ID:
  00=Post, 01=SetupServer, 02=Runtime, 03=DIM, 04=SetupClient, 05=RemoteClient
  06=DMI Data, 07=GreenPC, 08=Interface, 09=MP, 0A=NoteBook
  0B=Int-10, 0C=ROM-ID, 0D=Int-13, 0E=OEM Logo, 0F=ACPI Table
  10=ACPI AML, 11=P6 Micro Code, 12=Configuration, 13=DMI Code, 14=System Health
  15-1F=User defined, 20=PCIAddOnROM, 21=Multilanguage
  30=Font DataBase, 31=OEM Logo Data, 32=Graphic Logo Code, 33=Graphic Logo Data
  34=Action Logo Code, 35=Action Logo Data, 36=Virus, 37-3F=User denfined
Position# is the position of the module in the list.
Instance# is the instance of a particular type of module.
/U specifies that the module is to be placed as uncompressed.
/M <M#1> <M#2> sets the Destination Address or the PCI Device and Vendor ID's
   depending on the module type. The default is 0. M#1 is the Destination
   Segment or the Device ID. M#2 is the Destination Offset or the Vendor ID.
*/

#define BYTSPERLINE 16	/* fuer Hexdump */

static void hexdump(uchar *Buff, ulong len)
{
	int i;
	ulong done;

	done =0;
	while (done < len) {
		printf("%04x ", done);
		for (i =0; i< BYTSPERLINE; i++) {
			if ((done+i) < len) printf(" %02x", Buff[i]);
			else printf("   ");
		}
		printf(" ");
		for (i =0; i < BYTSPERLINE; i++) {
			if ((done+i) < len) printf("%c", isprint(Buff[i]) ? Buff[i] : '.');
		}
		printf("\n");
		done += BYTSPERLINE;
		Buff += BYTSPERLINE;
	}
}	/* hexdump */


int main(int argc, char *argv[]) {
	char * pInpName;
	FILE * fin;
	bios_head * pHead;
	ulong off;
	ushort mod;
	ulong thisbank, otherbank;
	uchar optE;
	uchar optX;

	printf("My AMIBIOS 10/10/94 Configuration Utility. Copyright (C) 2003 Herbert Oppmann.\n");
	printf("Version " VERSION " (" DATE ")\n\n");

	/* parse command line */
	optE = optX = 0;
	pInpName = NULL;
	while (--argc) {
		if (**(++argv) == '-') {
			switch (*(++*argv)) {
			case 'e': // extract modules (do not uncompress)
				optE =1;
				break;
			case 'x': // extract modules (uncompress)
				optX =1;
				break;
			default:
				printf("Unknown option '%c'.\n", **argv);
				exit(1);
			}
		} else {
			pInpName = *argv;
		}
	}
	if (pInpName == NULL) {
		pInpName = "BIOS.BIN";
	}

	fin = fopen(pInpName, "rb");
	if (fin == NULL) {
		printf("Can't open file '%s' for reading\n", pInpName);
		return 1;
	}
	if (fread(bimage, 1, 128*K, fin) != 128*K) {
		printf("Can't read 128KB from file '%s'\n", pInpName);
		return 1;
	}
	fclose(fin);
	for (off =0; off < (128*K-0x10); off += 0x10) {
		pHead = (bios_head *) &bimage[off];
		if ((pHead->id[5] == 'O') && (pHead->id[6] == 'S') && (pHead->id[7] == 'C')
		 && isdigit(pHead->date[0]) && isdigit(pHead->date[1]) && (pHead->date[2] == '/')
		 && isdigit(pHead->date[3]) && isdigit(pHead->date[4]) && (pHead->date[5] == '/')
		 && (pHead->date[6] == '9') && (pHead->date[7] == '4')) break;
	}
	if (off >= (128*K-0x10)) {
		printf("No AMIBIOS header found!\n");
		return 1;
	}
	if (strncmp("AMIBIOSC", pHead->id, 8) !=0) {
		printf("Crippled ");
	}
	printf("AMIBIOS header (dated %8.8s) found at %05lxh\n", pHead->date, off);
	if (off < 64*K) {
		thisbank = 0;
		otherbank = 64*K;
	} else {
		thisbank = 64*K;
		otherbank = 0;
	}
	printf("Module  Source   Dest   Comp  Uncmp  Content\n"
		   "--------------------------------------------\n");

	for (mod =0; mod < pHead->nMod; mod++) {
		int id;
		ulong src, dst;
		uchar bCompressed;
		uchar * pUnc;
		module_head * pModule;
		char *pID;

		id  = pHead->m[mod].to & 0xF;
		src = pHead->m[mod].from + ((pHead->m[mod].to & 0x80) ? otherbank : thisbank);
		dst = pHead->m[mod].to & 0xFF00;
		printf("    %2d  %05lXh  %04lXh  ", id, src, dst);

		pModule = (module_head *) &bimage[src];
		nCompSize = pModule->compsize;	// includes sizeof(module_head)
		nOrigSize = pModule->origsize;
		bCompressed = (nCompSize <= 64*K) && (nOrigSize <= 64*K) && (nCompSize <= nOrigSize);
		if (bCompressed) {
        	uchar *puc;
			printf("%04lXh", nCompSize);
			pUnc = malloc(nOrigSize);
			if (pUnc == NULL) {
				printf("Out of memory!\n");
				exit(1);
			}
			p_compressed = (uchar *)(pModule+1);
			p_uncompressed = pUnc;
			decode(nOrigSize);

#if 0
			// test if my compression leads to same result
			nCompSize = 0;
			nOrigSize = pModule->origsize;
			p_compressed = puc = malloc(nOrigSize);
			p_uncompressed = pUnc;
            encode();
			if (nCompSize != pModule->compsize-sizeof(struct module_head)-1) {
				ulong max;
                if (nCompSize > pModule->compsize-sizeof(struct module_head)-1)
                	max = nCompSize;
                else max= pModule->compsize-sizeof(struct module_head)-1;
				printf("LEN!\nOriginal:\n");
				hexdump((uchar *)(pModule+1), max);
				printf("\nRe-compressed:\n");
				hexdump(puc, max);
			} else if (memcmp((uchar *)(pModule+1), puc, nCompSize) != 0) {
				printf("CONTENT!\nOriginal:\n");
				hexdump((uchar *)(pModule+1), nCompSize);
				printf("\nRe-compressed:\n");
				hexdump(puc, nCompSize);
			}
			free(puc);
			nOrigSize = pModule->origsize;
#endif
		} else {
			printf("-----");
			nCompSize = nOrigSize = ((src < 64*K) ? 64*K : 128*K) - src;
			pUnc = (uchar *) pModule;
		}
		printf("  %04lXh", nOrigSize);
		// try to ID this module
			   if ((strncmp((const char *)pUnc, "AMIBIOSP", 8))==0) {
				pID = "POST";
		} else if ((strncmp((const char *)pUnc, "AMIBIOSS", 8))==0) {
				pID = "Setup";
		} else if ((nOrigSize >= 32*K)
				&& (strncmp((const char *)pUnc+(nOrigSize-32*K), "AMIBIOS W", 9)==0)) {
				pID = "Runtime";
		} else if ((strncmp((const char *)pUnc, "AMIBIOS W", 9))==0) {
				pID = "INIT";
		} else if ((strncmp((const char *)pUnc+5, "AMIBIOSD", 8))==0) {
				pID = "DIM";
		} else {
				pID = "";
		}

		printf("  %s\n", pID);
		if (optE || optX) {
			char fname[64];
			char tp;
			FILE *fout;
			tp = (optE && bCompressed) ? 'c' : 'u';
			if (pID[0] == '\0') {
				sprintf(fname, "%05lX_%c.bin", src, tp);
			} else {
				sprintf(fname, "%s_%c.bin", pID, tp);
			}
			fout = fopen(fname, "wb");
			if (optE) {
				if (fwrite(pModule, 1, nCompSize, fout) != nCompSize) {
					printf("Can't write!\n");
					exit(1);
				}
			} else { // optX
				if (fwrite(pUnc, 1, nOrigSize, fout) != nOrigSize) {
					printf("Can't write!\n");
					exit(1);
				}
			}
			fclose(fout);
		}
		if (bCompressed) {
			free(pUnc);
		}
	}
	printf("Done\n");
	return 0;
} /* main */

/*--------------------------------------------------------------------------*/
/*  Copyright (C) Herbert Oppmann 2003. All Rights Reserved.                */
/*--------------------------------------------------------------------------*/

